function [coeffs,fun] = dirac(N,M,alpha,kappa)
%DIRAC    QEP from Dirac operator.
%  [COEFFS,FUN] = NLEVP('dirac',N,M,ALPHA,KAPPA) generates the
%  coefficient matrices of a quadratic matrix polynomial
%  lambda^2*A + lambda*B + C of dimension N + M.  The spectrum
%  of this matrix polynomial is the second order spectrum of the
%  Dirac operator with an electric Coulombic potential, relative
%  to the subspace generated by the Hermite functions of odd order.
%  The potential is attractive and of strength ALPHA where
%  -SQRT(3)/2 < ALPHA < 0.
%  The constant KAPPA (which should be an integer) corresponds
%  to a radially symmetric decomposition of the space into partial wave
%  subspaces.  In this decomposition, the operator acts on
%  L^2(0,\infty)xL^2(0,\infty). The matrix truncation is found by
%  picking N Hermite functions in the first component and M in the second
%  component.
%  By default N = M = 40, KAPPA = -1, and ALPHA = -1/2.
%  The matrices are returned in a cell array: COEFFS = {C, B, A}.
%  FUN is a function handle to evaluate the monomials 1,lambda,lambda^2
%  and their derivatives.
%  This problem has the properties pep, qep, real, symmetric,
%  parameter-dependent, scalable.

%  This function is unvectorized and hence very slow for large N + M.

% Reference:
% L. Boulton and N. Boussaid, Non-variational computation of the eigenstates
% of Dirac operators with radially symmetric potentials,
% LMS J. Comput. Math., 13:10-32, 2010.

if nargin < 1 || isempty(N), N = 40; end
if nargin < 2 || isempty(M), M = N; end
if nargin < 3 || isempty(alpha), alpha = -1/2; end
if nargin < 4, kappa = -1; end

% Assemble matrices

MIT = eye(N+M,N+M);
MLT = MIT; MQT = MIT;
indi = [0:N-1,0:M-1];
for l = 1:N, for m = 1:N
  MLT(l,m) = LTR(indi(l),1,indi(m),1,kappa,alpha);
  MQT(l,m) = QTR(indi(l),1,indi(m),1,kappa,alpha);
end, end
for l = N+1:N+M, for m = 1:N
  MLT(l,m) = LTR(indi(l),2,indi(m),1,kappa,alpha);
  MQT(l,m) = QTR(indi(l),2,indi(m),1,kappa,alpha);
end, end
for l = 1:N, for m = N+1:N+M
  MLT(l,m) = LTR(indi(l),1,indi(m),2,kappa,alpha);
  MQT(l,m) = QTR(indi(l),1,indi(m),2,kappa,alpha);
end, end
for l = N+1:N+M, for m = N+1:N+M
  MLT(l,m) = LTR(indi(l),2,indi(m),2,kappa,alpha);
  MQT(l,m) = QTR(indi(l),2,indi(m),2,kappa,alpha);
end; end;
% Symmetrize (already symmetric up to roundoff).
MQT = (MQT + MQT')/2; MLT = (MLT + MLT')/2; MIT = (MIT + MIT')/2;
coeffs = {MQT,-2*MLT,MIT};

fun = @(lam) nlevp_monomials(lam,2);

%%%%%%%% The matrix coefficients are generated by the following functions

% Linear term

function T = LTR(k,pk,j,pj,kappa,alp)
if (pk == 1 && pj == 1), T = delta(k,j)+alp*R3(k,j); end;
if (pk == 2 && pj == 1), T = -R2(k,j)+kappa*R3(k,j); end;
if (pk == 1 && pj == 2), T = R2(k,j)+kappa*R3(k,j); end;
if (pk == 2 && pj == 2), T = -delta(k,j)+alp*R3(k,j); end;

%%%%%

% Quadratic term

function T = QTR(k,pk,j,pj,kappa,alp)
if (pk == 1 && pj == 1)
  T = delta(k,j)+2*alp*R3(k,j)+alp^2*R6(k,j)+R4(k,j)+kappa*R5(k,j)+...
    kappa*R5(j,k)+kappa^2*R6(k,j);
end
if (pk == 2 && pj == 1)
  T = -R2(k,j)-alp*R5(k,j)+2*kappa*alp*R6(k,j)-R2(j,k)+alp*R5(j,k);
end
if (pk == 1 && pj == 2)
  T = -R2(j,k)-alp*R5(j,k)+2*kappa*alp*R6(j,k)-R2(k,j)+alp*R5(k,j);
 end;
if (pk == 2 && pj == 2)
  T = R4(k,j)-kappa*R5(k,j)-kappa*R5(j,k)+kappa^2*R6(k,j)+delta(k,j)-...
    2*alp*R3(k,j)+alp^2*R6(j,k);
end

%%%%%%%
function de = delta(k,j)
de = 0;
if k == j, de = 1; end

%%%%%%%%%
function T = R2(k,j)
l = 1:j; Pj = prod(ones(size(l))+ones(size(l))./l/2);
m = 1:k; Pk = prod(ones(size(m))+ones(size(m))./m/2);
T = 4*(-1)^(k-j+1)*(k-j)/sqrt(pi)/(2*k-2*j-1)/(2*k-2*j+1)*sqrt(Pj*Pk);

%%%%%%%%%
function T = R3(k,j)
l = 1:j; Pj = prod(ones(size(l))+ones(size(l))./l/2);
l = 1:k; Pk = prod(ones(size(l))+ones(size(l))./l/2);
t = zeros(k+1,1);
for m = 0:k
    l = 1:m; Pm = prod(ones(size(l))+ones(size(l))./l/2);
    t(m+1) = Pm/(2*m-2*j-1)/(2*m+1);
end;
T = sqrt(Pj/Pk)*2*(-1)^(k-j+1)/sqrt(pi)*sum(t);

%%%%%%%%%%%%%%
function T = R4(k,j)
T = 0;
if j == k-1
  T = -sqrt(2*k+1)*sqrt(2*k)/2;
end;
if j == k
  T = (4*k+3)/2;
end;
if j == k+1
  T = -sqrt(2*k+2)*sqrt(2*k+3)/2;
end;


%%%%%%%%%%%%%%%%
function T = R5(k,j)
T = 0;
if  k < j
l = 1:j; Pj = prod(ones(size(l))+ones(size(l))./l/2);
l = 1:k; Pk = prod(ones(size(l))+ones(size(l))./l/2);
T = (-1)^(j-k)*2*sqrt(Pk/Pj);
end
if k == j
  T = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%
function T = R6(k,j)
l = 1:j; Pj = prod(ones(size(l))+ones(size(l))./l/2);
l = 1:k; Pk = prod(ones(size(l))+ones(size(l))./l/2);
if j <= k
T = (-1)^(k-j)*2*sqrt(Pj/Pk);
end;
 if j > k
T = (-1)^(j-k)*2*sqrt(Pk/Pj);
end
