function [coeffs,fun,sol] = gen_hyper2(n,E,V)
%GEN_HYPER2    Hyperbolic QEP constructed from prescribed eigenpairs.
%  [COEFFS,FUN,SOL] = NLEVP('gen_hyper2',N) generates an N-by-N
%  hyperbolic quadratic matrix polynomial
%  Q(lambda) = lambda^2*A + lambda*B + C
%  with eigenvalues and eigenvectors returned in SOL.EVAL and SOL.EVEC.
%  [COEFFS,FUN,SOL] = NLEVP('gen_hyper2',E,V) generates Q(lambda)
%  with eigenvalues and eigenvectors E and V, where E must be
%  a vector of length 2*N of real numbers ordered so that
%  min(E(1:N)) > max(E(N+1:2*N)) and V is a N-by-2*N matrix
%  satisfying V*[EYE(N) 0; 0 -EYE(N)*V' = 0.
%  The default value is N = 15.
%  Defaults:  E = N*randn(2*N,1), V = [V1 V1*U], where V1 = RANDN(N)
%  and U = ORTH(RANDN(N)).
%  The matrices are returned in a cell array: COEFFS = {C, B, A}.
%  FUN is a function handle to evaluate the monomials 1,lambda,lambda^2
%  and their derivatives.
%  This problem has the properties pep, qep, real, symmetric, hyperbolic,
%  parameter-dependent, scalable, solution.

%  References:
%  M. Al-Ammari and F. Tisseur, Hermitian matrix polynomials with real
%    eigenvalues of definite type. Part I: Classification. Technical
%    report, 2010.  In press, corrected proof.
%  C.-H. Guo, N. J. Higham, and F. Tisseur, Detecting and solving
%    hyperbolic quadratic eigenvalue problems. SIAM J. Matrix Anal.
%    Appl., 30(4):1593-1613, 2009.

if nargin < 1, n = 15; E = []; end
if nargin == 1
   if length(n) > 1
      E = n; n = length(E)/2;
   else
      E = [];
   end
end
if nargin == 2, V = E; E = n; n = length(E)/2; end
if n ~= round(n), error('E must have an even number of elements.'), end

if nargin == 2
   if ~all(size(V) == [n 2*n])
      error('Third input argument has incorrect size.')
   end
   V1 = V(:,1:n);
   V2 = V(:,n+1:2*n);
   if norm(V1*V1'-V2*V2',1) > n*eps*(norm(V1,1)^2 + norm(V2,1)^2)
      error('Incorrect set of eigenvectors.')
   end
else
   V1 = orth(randn(n)); % Arbitrary, but orth. leads to better conditioned A.
   V2 = gallery('qmult',V1',1)';
end

if ~isempty(E)
   E1 = E(1:n);
   E2 = E(n+1:2*n);
   if min(E1) < max(E2) || ~isreal(E)
      error('Incorrect set of eigenvalues.')
   end
else
   gap = 0;
   while gap < n*eps
     E = sort(n*randn(2*n,1));
     E2 = E(1:n); E1 = E(n+1:2*n);
     gap = abs(E1(n)-E2(1));
   end
end

Ainv = V1*diag(E1)*V1'-V2*diag(E2)*V2'; % Ainv is positive definite.
Ainv = (Ainv+Ainv')/2; % To ensure symmetry.
A = inv(Ainv);
B = -A*(V1*diag(E1.^2)*V1'-V2*diag(E2.^2)*V2')*A;
B = .5*(B+B');
C = -A*(V1*diag(E1.^3)*V1'-V2*diag(E2.^3)*V2')*A + B*Ainv*B;
C = .5*(C+C');
coeffs = {C, B, A};

fun = @(lam) nlevp_monomials(lam,2);

sol.evec = [V1 V2];
sol.eval = [E1; E2];
