function names = query(varargin)
%QUERY         Query helper function for NLEVP.
%  See HELP NLEVP for syntax.

% All properties recognized by the collection.
properties={...
    'rep'
    'pep'
    'qep'
    'nep'
    'nonregular'
    'nonsquare'
    'real'
    'symmetric'
    'hermitian'
    'hyperbolic'
    'elliptic'
    'overdamped'
    'proportionally-damped'
    't-palindromic'
    '*-palindromic'
    't-anti-palindromic'
    '*-anti-palindromic'
    't-even'
    '*-even'
    't-odd'
    '*-odd'
    'gyroscopic'
    'bvd'
    'sparse'
    'parameter-dependent'
    'scalable'
    'solution'};

if nargin == 1 && strcmpi(varargin{1},'properties')
   names = properties; return
end

% COINCIDENCE is an array with (i,j) element 1 if problem i has property j
% and 0 otherwise.

coincidence = zeros(1,length(properties));  % Will add rows below.
problems = [];

vararg1 = varargin{1};
quick_return = false;

% Set up problem properties.
% Put the problems in alphabetical order by name.

add_problem('acoustic_wave_1d','pep','qep','symmetric','*-even',...
            'parameter-dependent','scalable')
add_problem('acoustic_wave_2d','pep','qep','symmetric','*-even',...
            'parameter-dependent','scalable')
add_problem('bicycle','pep','qep','real','parameter-dependent')
add_problem('bilby','pep','qep','real','parameter-dependent')
add_problem('butterfly','pep','real','parameter-dependent','T-even',...
            'scalable')
add_problem('cd_player','pep','qep','real')
add_problem('closed_loop','pep','qep','real','parameter-dependent')
add_problem('concrete','pep','qep','symmetric','parameter-dependent',...
            'sparse')
add_problem('damped_beam','pep','qep','real','symmetric','scalable')
add_problem('dirac','pep','qep','real','symmetric',...
            'parameter-dependent','scalable');
add_problem('fiber','nep','sparse','solution')
add_problem('foundation','pep','qep','symmetric','sparse')
add_problem('gen_hyper2','pep','qep','real','symmetric','hyperbolic',...
            'parameter-dependent','scalable','solution')
add_problem('gun','nep','sparse')
add_problem('hadeler','nep','real','symmetric','scalable');
add_problem('intersection','pep','qep','real')
add_problem('hospital','pep','qep','real')
add_problem('loaded_string','rep','real','symmetric','parameter-dependent',...
            'scalable')
add_problem('metal_strip','pep','qep','real')
add_problem('mobile_manipulator','pep','qep','real')
add_problem('omnicam1','pep','qep','real')
add_problem('omnicam2','pep','qep','real')
add_problem('orr_sommerfeld','pep','parameter-dependent','scalable')
add_problem('pdde_stability','pep','qep','scalable','parameter-dependent',...
             'sparse','symmetric')
add_problem('plasma_drift','pep')
add_problem('power_plant','pep','qep','symmetric','parameter-dependent')
add_problem('qep1','pep','qep','real','solution')
add_problem('qep2','pep','qep','real','solution')
add_problem('qep3','pep','qep','real','parameter-dependent','solution')
add_problem('qep4','pep','qep','nonregular','nonsquare','real','solution')
add_problem('railtrack','pep','qep','t-palindromic','sparse')
add_problem('railtrack2','pep','qep','t-palindromic','sparse',...
            'scalable','parameter-dependent')
add_problem('relative_pose_5pt','pep','real')
add_problem('relative_pose_6pt','pep','qep','real')
add_problem('schrodinger','pep','qep','real','symmetric','sparse');
add_problem('shaft','pep','qep','real','symmetric','sparse');
add_problem('sign1','pep','qep','hermitian','parameter-dependent','scalable');
add_problem('sign2','pep','qep','hermitian','parameter-dependent','scalable');
add_problem('sleeper','pep','qep','real','scalable','symmetric',...
            'proportionally-damped','solution')
add_problem('speaker_box','pep','qep','real','symmetric');
add_problem('spring','pep','qep','real','symmetric',...
            'proportionally-damped','parameter-dependent','scalable');
add_problem('spring_dashpot','pep','qep','real','parameter-dependent',...
            'scalable')
add_problem('surveillance','pep','qep','real','nonsquare','nonregular')
add_problem('wing','pep','qep','real')
add_problem('wiresaw1','pep','qep','real','t-even','gyroscopic',...
            'parameter-dependent','scalable')
add_problem('wiresaw2','pep','qep','real','parameter-dependent','scalable')

if quick_return == true, return, end

if strcmpi(vararg1,'problems')
   names = problems'; return
end

columns = zeros(1,0);
for i=1:nargin
    if ~ischar(varargin{i}), error('Properties must be strings');end
    foundproperty = false;
    for j=1:length(properties),
        if strcmpi(varargin{i},properties{j}),
            foundproperty = true;
            columns = [columns,j];
            break;
        end
    end
    if ~foundproperty
        warning('NLEVP:unknown_property','Unknown property');
    end
end
rows = all(coincidence(:,columns),2);

if nargout == 0 && sum(rows) == 0
    disp('There is no problem in the collection matching your properties.');
end
[names{1:sum(rows)}] = problems{rows};

       % Nested function
       function add_problem(name,varargin)
       %ADD_PROBLEM   Adds new problem to data structures.

       % NJH: Quick return business is slightly inelegant, but from
       % within ADD_PROBLEM can only return to QUERY, not NLEVP.  Also
       % didn't want to add test after each call to ADD_PROBLEM in QUERY.

       if quick_return==true, return, end
       if strcmpi(name,vararg1)
          % Quick return if properties for a matrix requested.
          names = varargin';
          quick_return = true;
       end

       n = length(problems)+1;
       problems{n} = name;
       match = 0;
       for i = 1:length(varargin)
          prop = varargin{i};
          for j = 1:length(properties)
              if strcmpi(prop, properties{j})
                 match = j;
                 break
              end
          end
          if match == 0
             error(['Internal error: Property ' prop ' is illegal'])
          end
          coincidence(n,match) = 1;
       end
       end % of function
names = names(:);
end
