function [coeffs,fun] = sign2(n,a)
%SIGN2  QEP from rank-1 perturbation of 2*sin(x) + sign(x) operator.
%  [COEFFS,FUN] = NLEVP('sign2',N,MU) generates the coefficient matrices
%  of a quadratic matrix polynomial lambda^2*A + lambda*B + C of
%  dimension 2*N+1. The spectrum  of this matrix polynomial is the
%  second order spectrum of a rank one perturbation of the operator of
%  multiplication by the symbol 2*sin(x) + sign(x) acting on
%  L^2(-\pi,\pi), with respect to the Fourier basis:
%  e^{-iNx},\ldots,1,\ldots e^{iNx}.
%  The strength of the perturbation is the real parameter MU.
%  By default N = 40 and A = 0.
%  The matrices are returned in a cell array: COEFFS = {C, B, A}.
%  FUN is a function handle to evaluate the monomials 1,lambda,lambda^2
%  and their derivatives.
%  This problem has the properties pep, qep, hermitian, parameter-dependent,
%  scalable.

%  Reference:
%  L. Boulton, Non-variational approximation of discrete eigenvalues of
%  self-adjoint operators, IMA J. Numer. Anal. 27 (2007), pp. 102-121.

if nargin < 1 || isempty(n), n = 40; end
if nargin < 2, a = 0; end

% Fourier coeff of the symbol.
% fc(:,1) are the Fourier coefficients of the symbol.
% fc(:,2) are the Fourier coefficients of the square of the symbol.

fc = zeros(2*n+1,2);
for k=2:2:2*n
    fc(k,1) = 1i*4/(pi*(k-1));
    fc(k,2) = -16/(pi*(4-(k-1)^2));
end;
fc(3,1) = -1i/2;
fc(1,2) = 9/2;
fc(5,2) = -1/4;

% Matrix truncations.
K = zeros(2*n+1);
K(n+1,n+1) = a;
MK = zeros(2*n+1);
B = toeplitz(fc(1:2*n+1,1));
MK(:,n+1) = B(:,n+1);
Q = toeplitz(fc(1:2*n+1,2));
A0 = Q + a*MK + a*MK' + K^2;
A1 = -2*(B+K);
A2 = eye(2*n+1);
coeffs = {A0, A1, A2};
fun = @(lam) nlevp_monomials(lam,2);
